"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getAliasEvaluator;

var _path = _interopRequireDefault(require("path"));

var _evaluator = _interopRequireDefault(require("stylus/lib/visitor/evaluator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param {Object|false} aliases
 * @param {boolean} resolveTilde
 *
 * @returns {Function<AliasEvaluator>|boolean}
 */
function getAliasEvaluator(aliases, resolveTilde) {
  let nodeModulesPath = null;
  let aliasList = null;

  function resolveAlias(node) {
    const importPath = node.string;
    const firstChar = importPath[0];

    if (firstChar === '/') {
      return importPath;
    }

    if (nodeModulesPath === null) {
      nodeModulesPath = getNodeModulesPath();
    }

    if (node.filename.indexOf(nodeModulesPath) === 0) {
      return importPath;
    }

    if (aliases) {
      if (aliasList === null) {
        aliasList = getAliasList(aliases);
      }

      for (const entry of aliasList) {
        if (entry.alias === importPath) {
          return entry.path;
        } else if (!entry.exact && importPath.indexOf(entry.aliasRoot) === 0) {
          return _path.default.join(entry.path, importPath.slice(entry.aliasRoot.length));
        }
      }
    }

    if (resolveTilde && firstChar === '~') {
      return _path.default.join(nodeModulesPath, importPath.slice(1));
    }

    return importPath;
  }

  return class AliasEvaluator extends _evaluator.default {
    visitImport(imported) {
      const node = this.visit(imported.path).first;

      if (typeof node.string === 'string' && node.string !== '') {
        node.string = resolveAlias(node);
      }

      return super.visitImport(imported);
    }

  };
}

function getNodeModulesPath() {
  return _path.default.join(process.cwd(), 'node_modules');
}

function getAliasList(aliases) {
  const aliasList = [];

  if (typeof aliases !== 'object') {
    return aliasList;
  }

  for (let [alias, aliasPath] of Object.entries(aliases)) {
    let exact = false;

    if (alias.slice(-1) === '$') {
      exact = true;
      alias = alias.slice(0, -1);
    }

    aliasList.push({
      alias,
      aliasRoot: alias + '/',
      path: aliasPath.replace(/[/\\]+$/, ''),
      exact
    });
  }

  return aliasList;
}